# The LearningOnline Network with CAPA
# Tags Default Definition Module 
#
# $Id: londefdef.pm,v 1.106.2.2 2003/03/17 17:18:50 albertel Exp $
# 
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# Copyright for TtHfunc and TtMfunc by Ian Hutchinson. 
# TtHfunc and TtMfunc (the "Code") may be compiled and linked into 
# binary executable programs or libraries distributed by the 
# Michigan State University (the "Licensee"), but any binaries so 
# distributed are hereby licensed only for use in the context
# of a program or computational system for which the Licensee is the 
# primary author or distributor, and which performs substantial 
# additional tasks beyond the translation of (La)TeX into HTML.
# The C source of the Code may not be distributed by the Licensee
# to any other parties under any circumstances.
#
#
# last modified 06/26/00 by Alexander Sakharuk
# 11/6,11/30,02/01/01,5/4 Gerd Kortemeyer
# 01/18 Alex Sakharuk

package Apache::londefdef; 

use Apache::lonnet;
use strict;
use Apache::lonxml;
use Apache::File();
use Image::Magick;

BEGIN {

    &Apache::lonxml::register('Apache::londefdef',('a','abbr','acronym','address','allow','applet','area','b','base','basefont','bgo','bgsound','big','blink','blockquote','blankspace','body','br','button','caption','center','cite','code','col','colgroup','dd','del','dfn','dir','div','dl','dt','em','embed','externallink','fieldset','font','form','frame','frameset','h1','h2','h3','h4','h5','h6','head','hr','html','i','iframe','img','input','ins','insert','isindex','kbd','keygen','label','layer','legend','li','link','m','map','marquee','menu','meta','multicol','nobr','noembed','noframes','nolayer','noscript','object','ol','optgroup','option','output','p','param','pre','q','s','samp','select','server','small','spacer','span','strike','strong','sub','sup','table','tbody','td','textarea','tfoot','th','thead','title','tr','tt','tthoption','u','ul','var','wbr'));

}

#======================= TAG SUBROUTINES =====================
#-- <output>
sub start_output {
  my ($target) = @_;
  if ($target eq 'meta') { $Apache::lonxml::metamode--; }
  return '';
}
sub end_output {
  my ($target) = @_;
  if ($target eq 'meta') { $Apache::lonxml::metamode++; }
  return '';
}
#-- <m> tag
sub start_m {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
  my $currentstring = '';
  if ($target eq 'web') {
    $Apache::lonxml::prevent_entity_encode++;
    my $inside = &Apache::lonxml::get_all_text_unbalanced("/m",$parser);
    $inside ='\\documentstyle{article}'.$inside;
    &Apache::lonxml::debug("M is starting with:$inside:");
    my $eval=&Apache::lonxml::get_param('eval',$parstack,$safeeval);
    if ($eval eq 'on') {
      $inside=&Apache::run::evaluate($inside,$safeeval,$$parstack[-1]);
      #&Apache::lonxml::debug("M is evaulated to:$inside:");
    }
    $currentstring = &Apache::lontexconvert::converted(\$inside);
    if ($Apache::lontexconvert::errorstring) {
      &Apache::lonxml::warning("tth error: ".
			       $Apache::lontexconvert::errorstring);
      $Apache::lontexconvert::errorstring='';
    }
    #&Apache::lonxml::debug("M is ends with:$currentstring:");
  } elsif ($target eq 'tex') {
    $currentstring = &Apache::lonxml::get_all_text_unbalanced("/m",$parser);
    if ($currentstring=~/^(\s*\\\\\s*)*$/) {$currentstring = ' \vskip 0 mm ';}
  } else {
    my $inside = &Apache::lonxml::get_all_text_unbalanced("/m",$parser);
  }
  return $currentstring;
}
sub end_m {
  my ($target,$token) = @_;
  my $currentstring = '';
  if ($target eq 'web') {
    $Apache::lonxml::prevent_entity_encode--;
  } elsif ($target eq 'tex') {
    $currentstring = "";
  } elsif ($target eq 'meta') {
  }
  return $currentstring;
}

sub start_tthoption {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
  my $result;
  if ($target eq 'web') {
      my $inside = &Apache::lonxml::get_all_text("/tthoption",$parser);
      $inside=~s/^\s*//;
      if ($ENV{'browser.mathml'}) {
	  &tth::ttmoptions($inside);
      } else {
	  &tth::tthoptions($inside);
      }
  }
  return $result;
}

sub end_tthoption {
  my ($target,$token) = @_;
  my $result;
  return $result;
}

#-- <html> tag    
sub start_html {
    my ($target,$token) = @_;
    my $currentstring = '';
    if ($ENV{'browser.mathml'}) {
	&tth::ttminit();
	if ($ENV{'browser.unicode'}) {
	    &tth::ttmoptions('-L -u1');
	} else {
	    &tth::ttmoptions('-L -u0');
	}
    } else {
	&tth::tthinit();
	if ($ENV{'browser.unicode'}) {
	    &tth::tthoptions('-L -u1');
	} else {
	    &tth::tthoptions('-L -u0');
	}
    }
    if ($target eq 'web') {
	$currentstring = &Apache::lonxml::xmlbegin().
	    &Apache::lonxml::fontsettings();     
    } elsif ($target eq 'tex') {
	@Apache::londefdef::table = ();
	$currentstring .= '\documentclass[letterpaper]{article}
                                 \newcommand{\keephidden}[1]{}
                                 \renewcommand{\deg}{$^{\circ}$}
                                 \usepackage[dvips]{graphicx}
                                 \usepackage{epsfig}\usepackage{calc}
                                 \newenvironment{choicelist}{\begin{enumerate}}{\end{enumerate}}';
    }
    return $currentstring;
}
        sub end_html {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
		$currentstring = &Apache::lonxml::xmlend();
	    }
	   return $currentstring;
	}
#-- <head> tag
      sub start_head {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_head {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = &Apache::lonxml::registerurl(undef,$target).
                               $token->[2];    
            } 
	   return $currentstring;
	}
#-- <map> tag
      sub start_map {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_map {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <select> tag
      sub start_select {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_select {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <option> tag
      sub start_option {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_option {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <input> tag
      sub start_input {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_input {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <textarea> tag
      sub start_textarea {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_textarea {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <form> tag
      sub start_form {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_form {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <title> tag
      sub start_title {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring .= '\keephidden{' 
	    }
            if ($target eq 'meta') {
		$currentstring='<title>';
                &start_output();
            }
	   return $currentstring;
	}
        sub end_title {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } elsif ($target eq 'tex') {
              $currentstring .= '}';
	    }  
            if ($target eq 'meta') {
               &end_output();
               $currentstring='</title>';
            } 
	   return $currentstring;
	}
#-- <meta> tag
      sub start_meta {
	    my ($target,$token,$tagstack,$parstack,$parser) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	      my $args='';
	      if ( $#$parstack > -1 ) { $args=$$parstack[$#$parstack]; }
	      if ($args eq '') {
		&Apache::lonxml::get_all_text("/meta",$$parser[$#$parser]);
	      } else {
		$currentstring = $token->[4];
	      }
	    }
            if ($target eq 'meta') {
		unless ($token->[2]->{'http-equiv'}) {
		    my $name=$token->[2]->{'name'};
                    $name=~tr/A-Z/a-z/;
                    $name=~s/\s/\_/g;
                    if ($name) {
                       $currentstring='<'.$name.'>'.
                                         $token->[2]->{'content'}.
			              '</'.$name.'>';
		    }
                }
	    }
	   return $currentstring;
	}
      sub end_meta {
	my ($target,$token,$tagstack,$parstack,$parser) = @_;
	my $currentstring = '';
	if ($target eq 'web') {
	  my $args='';
	  if ( $#$parstack > -1 ) { $args=$$parstack[$#$parstack]; }
	  if ($args ne '') {
	    $currentstring = $token->[4];
	  }
	} 
	return $currentstring;
      }
#-- <body> tag
        sub start_body {
            my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	      if (!$Apache::lonxml::registered) {
		$currentstring.='<head>'.
		    &Apache::lonxml::registerurl(undef,$target).'</head>';
	      }
	      my $onLoad='';
	      foreach my $key (keys(%{$token->[2]})) {
		if ($key =~ /^onload$/i) {
		  $onLoad.=$token->[2]->{$key}.';';
		  delete($token->[2]->{$key});
		}
	      }
	      $token->[2]->{'onLoad'}=&Apache::lonxml::loadevents().
		                       ';'.$onLoad;
	      my $onUnload='';
	      foreach my $key (keys(%{$token->[2]})) {
		if ($key =~ /^onunload$/i) {
		  $onUnload.=$token->[2]->{$key}.';';
		  delete($token->[2]->{$key});
		}
	      }
	      $token->[2]->{'onUnload'}=&Apache::lonxml::unloadevents().
		                         ';'.$onUnload;

	      $currentstring .= '<'.$token->[1];
	      foreach (keys %{$token->[2]}) {
	      	$currentstring.=' '.$_.'="'.$token->[2]->{$_}.'"';
	      }
	      $currentstring.='>';
	      if ($ENV{'request.state'} ne 'published') {
		$currentstring.=(<<EDITBUTTON);
		<form method="post">
		<input type="submit" name="editmode" value="Edit" />
		</form>
EDITBUTTON
	      }
       	    } elsif ($target eq 'tex') {
              $currentstring = '\begin{document}';  
	    } 
	   return $currentstring;
	}
        sub end_body {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\end{document}';  
	    } 
	   return $currentstring;
	}
#-- <center> tag
        sub start_center {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\begin{center}';  
	    }  elsif ($target eq 'latexsource') {
              $currentstring = '\begin{center}';  
	    } 
	   return $currentstring;
	}
        sub end_center {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\end{center}';  
	    }  elsif ($target eq 'latexsource') {
              $currentstring = '\end{center}';  
	    } 
	   return $currentstring;
	}
#-- <b> tag
        sub start_b {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\textbf{';  
	    }  elsif ($target eq 'latexsource') {
              $currentstring = '\textbf{';  
	    } 
	   return $currentstring;
	}
        sub end_b {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '}';  

	    } elsif ($target eq 'latexsource') {
              $currentstring = '}';  
	    } 
	   return $currentstring;
	}
#-- <strong> tag
        sub start_strong {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\textbf{';  
	    } elsif ($target eq 'latexsource') {
              $currentstring = '\textbf{';  
	    } 
	   return $currentstring;
	}
        sub end_strong {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {

              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '}';  
	    }  elsif ($target eq 'latexsource') {
              $currentstring = '}';  
	    } 
	   return $currentstring;
	}
#-- <h1> tag
        sub start_h1 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\large \textbf{';
	    } elsif ($target eq 'meta') {
		$currentstring='<subject>';
                &start_output();
            }
           return $currentstring;
	}
        sub end_h1 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}}';
	    } elsif ($target eq 'meta') {
                &end_output();
		$currentstring='</subject>';
            } 
           return $currentstring;
	}
#-- <h2> tag
        sub start_h2 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\large \textbf{';
	    } 
           return $currentstring;
	}
        sub end_h2 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}}';
	    } 
           return $currentstring;
	}
#-- <h3> tag
        sub start_h3 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\large \textbf{';
	    } 
           return $currentstring;
	}
        sub end_h3 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}}';
	    } 
           return $currentstring;
	}
#-- <h4> tag
        sub start_h4 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\large \textbf{';
	    } 
           return $currentstring;
	}
        sub end_h4 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}}';
	    } 
           return $currentstring;
	}
#-- <h5> tag
        sub start_h5 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\large \textbf{';
	    } 
           return $currentstring;
	}
        sub end_h5 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}}';
	    } 
           return $currentstring;
	}
#-- <h6> tag
        sub start_h6 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\large \textbf{';
	    } 
           return $currentstring;
	}
        sub end_h6 {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}}';
	    } 
           return $currentstring;
	}
#--- <cite> tag
        sub start_cite {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\\textit{";
	    }  elsif ($target eq 'latexsource') {
		$currentstring .= "\\textit{";
	    } 
           return $currentstring;
	}
        sub end_cite {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= "}";
	    }  elsif ($target eq 'latexsource') {
		$currentstring .= "}";
	    } 
           return $currentstring;
	}
#-- <i> tag
        sub start_i {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\textit{';
	    }  elsif ($target eq 'latexsource') {
		$currentstring .= '\textit{';
	    } 
           return $currentstring;
	}
        sub end_i {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '}';
	    } 
           return $currentstring;
	}
#-- <address> tag
        sub start_address {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\\textit{";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "\\textit{";
	    } 
           return $currentstring;
	}
        sub end_address {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= "}";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "}";
	    }
           return $currentstring;
	}
#-- <dfn> tag
        sub start_dfn {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\\textit{";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "\\textit{";
	    } 
           return $currentstring;
	}
        sub end_dfn {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= "}";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "}";
	    } 
           return $currentstring;
	}
#-- <tt> tag
        sub start_tt {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\texttt{';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '\texttt{';
	    } 
           return $currentstring;
	}
        sub end_tt {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '}';
	    }
           return $currentstring;
	}
#-- <kbd> tag
        sub start_kbd {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\\texttt";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "\\texttt{";
	    } 
           return $currentstring;
	}
        sub end_kbd {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= "}";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "}";
	    } 
           return $currentstring;
	}
#-- <code> tag
        sub start_code {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\texttt{';
	    } 
           return $currentstring;
	}
        sub end_code {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}';
	    } 
           return $currentstring;
	}
#-- <em> tag
        sub start_em {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\emph{';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '\emph{';
	    } 
           return $currentstring;
	}
        sub end_em {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '}';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '}';
	    }  
           return $currentstring;
	}
#-- <q> tag
        sub start_q {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\\emph{";
	    }  elsif ($target eq 'latexsource') {
		$currentstring .= "\\emph{";
	    }
           return $currentstring;
	}
        sub end_q {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= "}";
	    } elsif ($target eq 'latexsource') {
		$currentstring .= "}";
	    }  
           return $currentstring;
	}
#-- <p> tag
        sub start_p {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\par ';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '{\par ';
	    } 
           return $currentstring;
	}
        sub end_p {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	        $currentstring .= '}';
            } elsif ($target eq 'latexsource') {
	        $currentstring .= '}';
            }
           return $currentstring;
	}
#-- <br> tag
        sub start_br {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		if ($$tagstack[-2] ne 'sub' && $$tagstack[-2] ne 'sup') {
		    $currentstring .= '\vskip 0.2 mm';
		}
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '\\';
	    } 
           return $currentstring;
	}
        sub end_br {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    }
           return $currentstring;
	}
#-- <big> tag
        sub start_big {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\large{';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '{\Large ';
	    }  
           return $currentstring;
	}
        sub end_big {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	        $currentstring .= '}';
            } elsif ($target eq 'latexsource') {
	        $currentstring .= '}';
            }
           return $currentstring;
	}
#-- <small> tag
        sub start_small {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '{\footnotesize ';
	    } elsif ($target eq 'latexsource') {
		$currentstring .= '{\footnotesize ';
	    } 
           return $currentstring;
	}
        sub end_small {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	        $currentstring .= '}';
            } elsif ($target eq 'latexsource') {
	        $currentstring .= '}';
            }
           return $currentstring;
	}
#-- <basefont> tag
      sub start_basefont {
	my ($target,$token) = @_;
	my $currentstring = '';
	if ($target eq 'web') {
	  $currentstring = $token->[4];     
	} 
	return $currentstring;
      }
      sub end_basefont {
	my ($target,$token) = @_;
	my $currentstring = '';
	if ($target eq 'web') {
	  $currentstring = $token->[4];     
	} 
	return $currentstring;
      }
#-- <font> tag
         sub start_font {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
            my $currentstring = '';
            if ($target eq 'web') {
	      my $face=&Apache::lonxml::get_param('face',$parstack,$safeeval);
	      if ($face=~/symbol/i) {$Apache::lonxml::prevent_entity_encode++;}
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_font {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
            my $currentstring = '';
            if ($target eq 'web') {
	      my $face=&Apache::lonxml::get_param('face',$parstack,$safeeval);
	      if ($face=~/symbol/i) {$Apache::lonxml::prevent_entity_encode--;}
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	} 
#-- <strike> tag
        sub start_strike {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
	       &Apache::lonxml::startredirection();
	    } 
           return $currentstring;
	}
        sub end_strike {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	       $currentstring=&Apache::lonxml::endredirection();
	       $currentstring=~s/(\S)(\s+)(\S)/$1}$2\\underline{$3/g;
	       $currentstring=~s/^\s*(\S)/\\underline{$1/;
	       $currentstring=~s/(\S)\s*$/$1}/;	 	       
            }
           return $currentstring;
	}
#-- <s> tag
        sub start_s {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
	       &Apache::lonxml::startredirection();
	    } 
           return $currentstring;
	}
        sub end_s {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	       $currentstring=&Apache::lonxml::endredirection();
	       $currentstring=~s/(\S)(\s+)(\S)/$1}$2\\underline{$3/g;
	       $currentstring=~s/^\s*(\S)/\\underline{$1/;
	       $currentstring=~s/(\S)\s*$/$1}/;	
            }
           return $currentstring;
	}
#-- <sub> tag
        sub start_sub {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\$_{ ";
	    } 
           return $currentstring;
	}
        sub end_sub {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	        $currentstring .= " }\$";
            }
           return $currentstring;
	}
#-- <sup> tag
        sub start_sup {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= "\$^{ ";
	    } 
           return $currentstring;
	}
        sub end_sup {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	        $currentstring .= " }\$";
            }
           return $currentstring;
	}
#-- <hr> tag
        sub start_hr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\vskip 0 mm \noindent\makebox[\textwidth/2 ][b]{\hrulefill}\vskip 0 mm ';
	    } 
           return $currentstring;
	}
        sub end_hr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	    } 
           return $currentstring;
	}
#-- <div> tag
        sub start_div {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } 
           return $currentstring;
	}
        sub end_div {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } 
           return $currentstring;
	}
#-- <a> tag
        sub start_a {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
	    }
           return $currentstring;
	}
        sub end_a {
	    my ($target,$token,$tagstack,$stackref) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
                my  $tempor_var = $stackref->[$#$stackref];
		if (index($tempor_var,'name') != -1 ) {
		    $tempor_var =~ s/name=([^,]*),/$1/g;
	        } elsif (index($tempor_var,'href') != -1 ) {
		    $tempor_var =~ s/href=([^,]*),/$1/g;
	        $currentstring .= " \\ref{$tempor_var}";
	        }
            }
           return $currentstring;
	}
#-- <li> tag
        sub start_li {
	    my ($target,$token,$tagstack,$stackref) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
                my  $tempor_var = $stackref->[$#$stackref];
                if (index($tempor_var,'circle') != -1 ) {
	          $currentstring .= " \\item[o] ";
	        } elsif (index($tempor_var,'square') != -1 ) {
	               $currentstring .= " \\item[$\Box$] ";
	        } elsif ($tempor_var ne '') { 
		       $_ = $tempor_var;
                       m/my\s*([^=]*)=/;
		       $currentstring .= " \\item[$1] ";
		} else {
		    $currentstring .= " \\item ";
	        }  
	    } 
	   return $currentstring;
	}
        sub end_li {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } 
	   return $currentstring;
	}
#-- <u> tag
        sub start_u {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
	       &Apache::lonxml::startredirection();
	    } 
           return $currentstring;
	}
        sub end_u {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
	       $currentstring=&Apache::lonxml::endredirection();
	       $currentstring=~s/(\S)(\s+)(\S)/$1}$2\\underline{$3/g;
	       $currentstring=~s/^\s*(\S)/\\underline{$1/;
	       $currentstring=~s/(\S)\s*$/$1}/;		
            }
           return $currentstring;
	}
#-- <ul> tag
        sub start_ul {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\begin{itemize}';  
	    } 
	   return $currentstring;
	}
        sub end_ul {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\end{itemize}';  
	    } 
	   return $currentstring;
	}
#-- <menu> tag
        sub start_menu {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\begin{itemize} ";  
	    } 
	   return $currentstring;
	}
        sub end_menu {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\end{itemize}";  
	    } 
	   return $currentstring;
	}
#-- <dir> tag
        sub start_dir {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\begin{itemize} ";  
	    } 
	   return $currentstring;
	}
        sub end_dir {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\end{itemize}";  
	    } 
	   return $currentstring;
	}
#-- <ol> tag
        sub start_ol {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\begin{enumerate}';  
	    } 
	   return $currentstring;
	}
        sub end_ol {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\end{enumerate}';  
	    } 
	   return $currentstring;
	}
#-- <dl> tag
        sub start_dl {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\begin{description}';  
	    } 
	   return $currentstring;
	}
        sub end_dl {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\end{description}';  
	    } 
	   return $currentstring;
	}
#-- <dt> tag
        sub start_dt {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = '\item[';  
	    } 
	   return $currentstring;
	}
        sub end_dt {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } elsif ($target eq 'tex') {
              $currentstring = ']';  
	    } 
	   return $currentstring;
	}
#-- <dd> tag
        sub start_dd {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_dd {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <table> tag
sub start_table {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
    my $currentstring = '';
    if ($target eq 'web') {
	$currentstring = $token->[4];     
    } elsif ($target eq 'tex') {
	my $aa = {};
	push @Apache::londefdef::table, $aa; 
	$Apache::londefdef::table[-1]{'row_number'} = -1;
	$Apache::londefdef::table[-1]{'output'} = ' \noindent \begin{tabular} ';
	my $border = &Apache::lonxml::get_param('border',$parstack,$safeeval,undef,1); 
	unless (defined $border) { $border = 0; }
	if ($border) { 
	    $Apache::londefdef::table[-1]{'hinc'} = '\hline '; 
	    $Apache::londefdef::table[-1]{'vinc'} = '&'; 
	    $Apache::londefdef::table[-1]{'vvinc'} = '|';
	} else {
	    $Apache::londefdef::table[-1]{'hinc'} = ''; 
	    $Apache::londefdef::table[-1]{'vinc'} = '&'; 
	    $Apache::londefdef::table[-1]{'vvinc'} = '';
	}
	my $width;
	foreach my $key (keys(%{$token->[2]})) {
	    if ($key =~ /^width$/i) {
		$width = &Apache::lonxml::get_param($key,$parstack,$safeeval,undef,1);
	    }
	}
	if (defined($width)) { $Apache::londefdef::table[-1]{'width'}=$width; }
    }
    return $currentstring;
}
        sub end_table {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
		my $inmemory = '';
		my $output = '';
                #construct header of the table
		my $header_of_table = '{'.$Apache::londefdef::table[-1]{'vvinc'};
		my $in;
		for ($in=0;$in<=$Apache::londefdef::table[-1]{'counter_columns'};$in++) {
		    $header_of_table .= $Apache::londefdef::table[-1]{'columns'}[$in].$Apache::londefdef::table[-1]{'vvinc'};
		}
		$header_of_table .= '}';
                #fill the table
		for ($in=0;$in<=$Apache::londefdef::table[-1]{'row_number'};$in++) {
		    $output .=  $Apache::londefdef::table[-1]{'rowdata'}[$in];
		    chop $output;
		    $output .= ' \\\\ ';
		}
                #define the length of the table cells
		my @lengthforoutput = split(/,/,$Apache::londefdef::table[-1]{'lengthrow'}[0]);
		my $how_many_columns = $#lengthforoutput + 1; #total number of columns in the table 
		my $filled_columns = 0; #number of columns with information about width
		my $available_space = ' ';
		foreach my $tempo_length (@{ $Apache::londefdef::table[-1]{'lengthrow'} }) {
		    my @length = split(/,/,$tempo_length);
		    for (my $ico=0;$ico<=$#lengthforoutput;$ico++) {
			$lengthforoutput[$ico] =~ m/(\d*\.?\d*)\s*(\w+)/;
			my $old_value = $1;
			my $old_unit = $2; 
			if ($old_unit eq 'cm') { 
			    $old_value = $old_value * 10;
			} elsif ($old_unit eq 'in') { 
			    $old_value = $old_value * 25.4;
			} elsif ($old_unit eq 'pt') {
			    $old_value = $old_value * 25.4/72.27;
			} elsif ($old_unit eq 'pc') {
			    $old_value = $old_value * 25.4/6.022;
			}
			$old_unit = 'mm';
 			$length[$ico] =~ m/(\d*\.?\d*)\s*(\w+)/;
			my $new_value = $1;
			my $new_unit = $2;
			if ($new_unit eq 'cm') { 
			    $new_value = $new_value * 10;
			} elsif ($old_unit eq 'in') { 
			    $new_value = $new_value * 25.4;
			} elsif ($old_unit eq 'pt') {
			    $new_value = $new_value * 25.4/72.27;
			} elsif ($old_unit eq 'pc') {
			    $new_value = $new_value * 25.4/6.022;
			}
			$new_unit = 'mm';
			if ($old_value < $new_value) {
			    $lengthforoutput[$ico] = $new_value.' mm';
			} else {
			    $lengthforoutput[$ico] = $old_value.' mm';
			}		 
		    }
		}
		for (my $ico=0;$ico<=$#lengthforoutput;$ico++) {
		    if (not $lengthforoutput[$ico]=~m/^\s*0\s*\w*\s*$/) {
			$filled_columns++;
			$available_space = $available_space.' - '.$lengthforoutput[$ico];
		    }
		}
		my $temp_file;
		my $filename = "/home/httpd/prtspool/$ENV{'user.name'}_$ENV{'user.domain'}_printout.tbl";
		if (-e $filename) {
		    $temp_file = Apache::File->new($filename); 
		    my @tbl_file_content = <$temp_file>;
		    my ($one,$two,$three) = split(/,/,$tbl_file_content[0]);
		        $how_many_columns+=$one-1;
		        $filled_columns+=$two;
		    if($three=~/\S/) {$available_space = $available_space.' - '.$three;}
		} else {
		    $temp_file = Apache::File->new('>>'.$filename); 
		}		
		print $temp_file "$how_many_columns,$filled_columns,$available_space\n";
		$output =~ s/\\parbox{TOBECHANGEDONNUMBER}{}/\\parbox{1 mm}{}/g;
		$output =~ s/\\parbox{TOBECHANGEDONNUMBER}/\\parbox{\$SpacePerColumn}/g;
		my @tagar = @$tagstack;
		my $signature = 1;
		for (my $ico=0;$ico<$#tagar;$ico++) {
		     if ($tagar[$ico] eq 'table') { $signature = 0; }
		}
		if ($signature) {
		    my $NumberEmptyLength = $how_many_columns - $filled_columns;		
		    my $SpacePerColumn = '(\textwidth '.$available_space.')/'.$NumberEmptyLength;
		    my $shorthand = ($filled_columns+1)*4;
		    $output =~ s/\$SpacePerColumn/$SpacePerColumn - $shorthand mm/g;
		}	
		$Apache::londefdef::table[-1]{'output'} .= $header_of_table.$output.$Apache::londefdef::table[-1]{'hinc'}.'\end{tabular}\vskip 0 mm ';
		if ($#Apache::londefdef::table > 0) {
		    $inmemory = $Apache::londefdef::table[-1]{'output'};
		    pop @Apache::londefdef::table;
		    $Apache::londefdef::table[-1]{'rowdata'}[$Apache::londefdef::table[-1]{'row_number'}] .= $inmemory
		} else {
		    $currentstring = $Apache::londefdef::table[-1]{'output'};
		    $currentstring =~ s/\\\\\s+\\\\/\\\\/g; 
		    pop @Apache::londefdef::table;
		    if (-e $filename) {
			unlink $filename;
		    }
		}
	    }
	   return $currentstring;
	}
#-- <tr> tag
        sub start_tr {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
		$Apache::londefdef::table[-1]{'row_number'}++;
		my $alignchar = substr(&Apache::lonxml::get_param('align',$parstack,$safeeval,undef,1),0,1);
		if ($alignchar ne '') {
		    push @ {$Apache::londefdef::table[-1]{'rows'} }, $alignchar;
		} else {
		    push @ {$Apache::londefdef::table[-1]{'rows'} }, 'l';
		}
		push ( @{ $Apache::londefdef::table[-1]{'rowdata'} }, $Apache::londefdef::table[-1]{'hinc'});
		$Apache::londefdef::table[-1]{'counter_columns'} = -1;
		$Apache::londefdef::table[-1]{'length'} = ''; 
	    } 
	   return $currentstring;
	}        
        sub end_tr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
		$currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
		push @{ $Apache::londefdef::table[-1]{'lengthrow'} },$Apache::londefdef::table[-1]{'length'};

	    }
	   return $currentstring;
	}
#-- <td> tag
        sub start_td {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
    } elsif ($target eq 'tex') {
		my $what_to_push = substr(&Apache::lonxml::get_param('align',$parstack,$safeeval,undef,1),0,1);
		if ($what_to_push eq '') {
		    $what_to_push = substr($Apache::londefdef::table[-1]{'rows'}[0],0,1);;
		}
		    push @{ $Apache::londefdef::table[-1]{'columns'} }, $what_to_push;
		$Apache::londefdef::table[-1]{'counter_columns'}++;
		&Apache::lonxml::startredirection();
;
	    } 
	   return $currentstring;
	}        
        sub end_td {
	    my ($target,$token) = @_;
            my $currentstring = '';
            my $tempolen = '';
            if ($target eq 'web') {
		$currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
		my $current_row = $Apache::londefdef::table[-1]{'row_number'};
		my $data=&Apache::lonxml::endredirection();
		if ($data=~m/width\s*=\s*(\d+\.?\d*\s*(mm|cm|in|pc|pt))/) {                 
                    $Apache::londefdef::table[-1]{'length'} .= $1.',';
		    $tempolen = $1;         
		} else {
		    if (length($data)<5) {
			$Apache::londefdef::table[-1]{'length'} .= '0 mm,';
			$tempolen = '6 mm';
		    } else {
			$Apache::londefdef::table[-1]{'length'} .= '0 mm,';
			$tempolen = 'TOBECHANGEDONNUMBER';
		    }
                }                                        
		@{ $Apache::londefdef::table[-1]{'rowdata'} }[$current_row] .= '\parbox{'.$tempolen.'}{'.$data.'} '.$Apache::londefdef::table[-1]{'vinc'};
	    }
	   return $currentstring;
	}
#-- <th> tag
        sub start_th {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
		my $what_to_push = substr(&Apache::lonxml::get_param('align',$parstack,$safeeval,undef,1),0,1);
		if ($what_to_push eq '') {
		    $what_to_push = substr($Apache::londefdef::table[-1]{'rows'}[0],0,1);;
		}
		    push @{ $Apache::londefdef::table[-1]{'columns'} }, $what_to_push;
		$Apache::londefdef::table[-1]{'counter_columns'}++;
		&Apache::lonxml::startredirection();
;
	    } 
	   return $currentstring;
	}        
        sub end_th {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
		$currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
		my $current_row = $Apache::londefdef::table[-1]{'row_number'};
		my $data=&Apache::lonxml::endredirection();
		if ($data=~m/width\s*=\s*(\d+\.*\d*\s*(mm|cm))/) {                 
                    $Apache::londefdef::table[-1]{'length'} .= $1.',';             
		} else {
		    $Apache::londefdef::table[-1]{'length'} .= '0 mm,'; 
                }                                                           
		@{ $Apache::londefdef::table[-1]{'rowdata'} }[$current_row] .= '\parbox{'.$1.'}{\textbf{'.$data.'}} '.$Apache::londefdef::table[-1]{'vinc'};
	    }
	   return $currentstring;
	}
#-- <img> tag
        sub start_img {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
	    my $src = &Apache::lonxml::get_param('src',$parstack,$safeeval,
						 undef,1);
            $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=$src;
            my $currentstring = '';
	    my $width_param = '';
	    my $height_param = '';
	    my $scaling = .3;

	    if ($target eq 'web') {
              $currentstring = $token->[4];
	    } elsif ($target eq 'tex') {
	      &image_replication($src);
	      $src=&Apache::lonnet::filelocation($Apache::lonxml::pwd[-1],$src);
              #if original gif/jpg/png file exist do following:
              if (-e $src) {          
                  #defines the default size of image
		  my $image = Image::Magick->new;
		  my $current_figure = $image->Read($src);
		  $width_param = $image->Get('width') * $scaling;;
		  $height_param = $image->Get('height') * $scaling;;
		  undef $image;
		  #do we have any specified size of the picture?
		  my $TeXwidth = &Apache::lonxml::get_param('TeXwidth',$parstack,$safeeval);
		  my $TeXheight = &Apache::lonxml::get_param('TeXheight',$parstack,$safeeval);
		  my $width = &Apache::lonxml::get_param('width',$parstack,$safeeval,
                                                         undef,1);
		  if ($TeXwidth ne '') {  
		      if ($TeXwidth=~/(\d+)\s*\%/) {
			  $width_param = $1*$ENV{'form.textwidth'}/100;
		      } else { 
			  $width_param = $TeXwidth;
		      }
		  } elsif ($TeXheight ne '') {
			  $width_param = $TeXheight/$height_param*$width_param;
		  } elsif ($width ne '') {
		      $width_param = $width*$scaling;      
		  }
		  my $file;
		  my $path;	
		  if ($src =~ m!(.*)/([^/]*)$!) {
		      $file = $2; 
		      $path = $1.'/'; 
		  } 
		  my $newsrc = $src;
		  $newsrc =~ s/\.(gif|jpg|png)$/.eps/i;
		  $file=~s/\.(gif|jpg|png)$/.eps/i;
		  #where can we find the picture?
		  if (-e $newsrc) {
                      #eps counterpart for image exist 
		      if ($path) {
			  $currentstring .= '\vskip 1 mm \noindent\graphicspath{{'.$path.'}}\includegraphics[width='.$width_param.' mm]{'.$file.'} ';
		      }
		  } else {
                      #there is no eps counterpart for image - check for ps one
		      $newsrc =~ s/\.eps$/\.ps/;
		      if (-e $newsrc) {
			  #ps counterpart for image exist 
			  $file =~ s/\.eps$/\.ps/;
			  if ($path) {
			      $currentstring .= '\vskip 1 mm \noindent\graphicspath{{'.$path.'}}\includegraphics[width='.$width_param.' mm]{'.$file.'} ';
			  }
		      } else {
			  #there aren't eps or ps - so create eps 
			  my $temp_file;
			  my $filename = "/home/httpd/prtspool/$ENV{'user.name'}_$ENV{'user.domain'}_printout.dat";
			  $temp_file = Apache::File->new('>>'.$filename); 
			  print $temp_file "$src\n";
			  $currentstring .= '\vskip 1 mm \graphicspath{{/home/httpd/prtspool/}}\includegraphics[width='.$width_param.' mm]{'.$file.'} ';
		      }
		  }
	      } else {
		  #original image file doesn't exist so check the alt attribute
		  my $alt = &Apache::lonxml::get_param('alt',$parstack,$safeeval,undef,1);
                  if ($alt) {
		      $currentstring .= ' '.$alt.' ';
		  } else {
                     #<allow> tag will care about replication 
		  }
	      }
	    }
	    return $currentstring;
	}
        sub end_img {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];
	    } elsif ($target eq 'tex') {
              $currentstring = '';
	    }
	   return $currentstring;
	}
#-- <applet> tag

        sub start_applet {
	    my ($target,$token) = @_;
              $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=   
                                        $token->[2]->{'code'};
              $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=
                                        $token->[2]->{'archive'};
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];
	    } elsif ($target eq 'tex') {
              $currentstring = " \\begin{figure} ";
	    } 
	   return $currentstring;
	}
sub end_applet {
    my ($target,$token) = @_;
    my $currentstring = '';
    if ($target eq 'web') {
	$currentstring = $token->[2];
    } elsif ($target eq 'tex') {
	$currentstring = " \\end{figure}";
    } 
    return $currentstring;
}

#-- <embed> tag

sub start_embed {    
    my ($target,$token) = @_;
    $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=
	$token->[2]->{'src'};
    my $currentstring = '';
    if ($target eq 'web') {
	$currentstring = $token->[4];
    } elsif ($target eq 'tex') {
	$currentstring = " \\begin{figure} ";  
    } 
    return $currentstring;
}
        sub end_embed {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\end{figure}";  
	    } 
	   return $currentstring;
	}

#-- <param> tag

        sub start_param {
	    my ($target,$token) = @_;
	    if ($token->[2]->{'name'} eq 'cabbase') {
                      $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=
					     $token->[2]->{'value'};
            }   
            $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=   
                                        $token->[2]->{'src'};
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\begin{figure} ";  
	    } 
	   return $currentstring;
	}
        sub end_param {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];     
	    } elsif ($target eq 'tex') {
              $currentstring = " \\end{figure}";  
	    } 
	   return $currentstring;
	}
#-- <allow> tag
sub start_allow {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
    $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=
	$token->[2]->{'src'};
    my $src = &Apache::lonxml::get_param('src',$parstack,$safeeval,undef,1);
    &image_replication($src);
    my $result;
    if ($target eq 'edit') {
	$result .=&Apache::edit::tag_start($target,$token);
	$result .=&Apache::edit::text_arg('File Spec:','src',$token,70);
	$result .=&Apache::edit::end_row();#.&Apache::edit::start_spanning_row();
    } elsif ($target eq 'modified') {
	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						     $safeeval,'src');
	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
    }
    return $result;
}

sub end_allow {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
    if ( $target eq 'edit') { return (&Apache::edit::end_table()); }
    return '';
}
#-- Frames
	sub start_frameset {
	  my ($target,$token) = @_;
	  my $currentstring = '';
	  if ($target eq 'web') { 
	    if (!$Apache::lonxml::registered) {
	      $currentstring.='<head>'.
		  &Apache::lonxml::registerurl(undef,$target).'</head>';
	    }
	    $currentstring .= $token->[4];
	  }
	  return $currentstring;
	}
        sub end_frameset {
	  my ($target,$token) = @_;
	  my $currentstring = '';
	  if ($target eq 'web') {
	    $currentstring = $token->[2];
	  }
	  return $currentstring;
	}
#-- <pre>
	sub start_pre {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[4];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\begin{verbatim}';
	    } 
           return $currentstring;
	}
        sub end_pre {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= $token->[2];
	    } elsif ($target eq 'tex') {
		$currentstring .= '\end{verbatim}';
	    }
           return $currentstring;
	}
#-- <insert>
	sub start_insert {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
		my $display = &Apache::lonxml::get_param('display',$parstack,$safeeval,undef,1);
	       $currentstring .= '<b>'.$display.'</b>';;
	    }
           return $currentstring;
	}
        sub end_insert {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= '';
	    }
           return $currentstring;
	}
#-- <externallink>
	sub start_externallink {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
		my $display = &Apache::lonxml::get_param('display',$parstack,$safeeval,undef,1);
	       $currentstring .= '<b>'.$display.'</b>';;
	    }
           return $currentstring;
	}
        sub end_externallink {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
	       $currentstring .= '';
	    }
           return $currentstring;
	}
#-- <blankspace heigth="">
        sub start_blankspace {
	    my ($target,$token,$tagstack,$parstack,$parser,$safeeval) = @_;
            my $currentstring = '';
            if ($target eq 'tex') {
		my $howmuch = &Apache::lonxml::get_param('heigth',$parstack,$safeeval,undef,1);
		$currentstring .= '\vskip '.$howmuch.' ';
	    }
           return $currentstring;
	}
        sub end_blankspace {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'tex') {
	       $currentstring .= '';
	    }
           return $currentstring;
	}
#-- <abbr> tag
      sub start_abbr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_abbr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <acronym> tag
      sub start_acronym {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_acronym {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <area> tag
      sub start_area {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_area {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <base> tag
      sub start_base {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_base {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <bdo> tag
      sub start_bdo {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_bdo {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <bgsound> tag
      sub start_bgsound {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_bgsound {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <blink> tag
      sub start_blink {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_blink {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <blockquote> tag
      sub start_blockquote {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_blockquote {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <button> tag
      sub start_button {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_button {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <caption> tag
      sub start_caption {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_caption {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <col> tag
      sub start_col {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_col {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <colgroup> tag
      sub start_colgroup {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_colgroup {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <del> tag
      sub start_del {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_del {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <fieldset> tag
      sub start_fieldset {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_fieldset {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <frame> tag
      sub start_frame {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_frame {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <iframe> tag
      sub start_iframe {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_iframe {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <ins> tag
      sub start_ins {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_ins {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <isindex> tag
      sub start_isindex {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_isindex {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <keygen> tag
      sub start_keygen {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_keygen {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <label> tag
      sub start_label {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_label {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <layer> tag
      sub start_layer {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_layer {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <legend> tag
      sub start_legend {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_legend {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <link> tag
      sub start_link {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_link {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <marquee> tag
      sub start_marquee {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_marquee {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <malticol> tag
      sub start_malticol {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_malticol {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <nobr> tag
      sub start_nobr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_nobr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <noembed> tag
      sub start_noembed {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_noembed {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <noframes> tag
      sub start_noframes {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_noframes {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <nolayer> tag
      sub start_nolayer {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_nolayer {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <noscript> tag
      sub start_noscript {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_noscript {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <object> tag
      sub start_object {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_object {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <optgroup> tag
      sub start_optgroup {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_optgroup {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <samp> tag
      sub start_samp {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_samp {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <server> tag
      sub start_server {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_server {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <spacer> tag
      sub start_spacer {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_spacer {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <span> tag
      sub start_span {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_span {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <tbody> tag
      sub start_tbody {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_tbody {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <tfoot> tag
      sub start_tfoot {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_tfoot {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <thead> tag
      sub start_thead {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_thead {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <var> tag
      sub start_var {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_var {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}
#-- <wbr> tag
      sub start_wbr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[4];     
	    } 
	   return $currentstring;
	}
        sub end_wbr {
	    my ($target,$token) = @_;
            my $currentstring = '';
            if ($target eq 'web') {
              $currentstring = $token->[2];    
            } 
	   return $currentstring;
	}

sub image_replication {
    my $src = shift;
    if (not -e '/home/httpd/html'.$src) {
	#replicates image itself
	&Apache::lonnet::repcopy('/home/httpd/html'.$src);
	#replicates eps or ps 
	my $newsrc = $src;
	$newsrc =~ s/\.(gif|jpg|png)$/.eps/;
	if (not-e $newsrc && &Apache::lonnet::repcopy('/home/httpd/html'.$newsrc) ne 'OK') {
	    $newsrc =~ s/\.ps$/\.eps/;
	    &Apache::lonnet::repcopy('/home/httpd/html'.$newsrc);
	}
    }
    return '';
}

1;
__END__
