# The LearningOnline Network with CAPA
#
# Page flip handler
#
# $Id: lonpageflip.pm,v 1.33 2002/12/06 14:06:53 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (Page Handler
#
# (TeX Content Handler
#
# 05/29/00,05/30 Gerd Kortemeyer)
# 08/30,08/31,09/06,09/14,09/15,09/16,09/19,09/20,09/21,09/23,
# 10/02 Gerd Kortemeyer)
#
# 10/03,10/05,10/06,10/07,10/09,10/10,10/11,10/16,10/17,
# 11/14,11/16,
# 10/01/01,05/01,05/28,07/05 Gerd Kortemeyer

package Apache::lonpageflip;

use strict;
use Apache::Constants qw(:common :http REDIRECT);
use Apache::lonnet();
use HTML::TokeParser;
use GDBM_File;

# ========================================================== Module Global Hash
  
my %hash;

sub addrid {
    my ($current,$new,$condid)=@_;
    unless ($condid) { $condid=0; }

	if ($current) {
	    $current.=','.$new;
        } else {
            $current=''.$new;
        }

    return $current;
}

sub fullmove {
    my ($rid,$mapurl,$direction)=@_;
    if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.'.db',
                        &GDBM_READER(),0640)) {
	($rid,$mapurl)=&move($rid,$mapurl,$direction);
        untie(%hash);
    }
    return($rid,$mapurl);
}

sub move {
    my ($rid,$mapurl,$direction)=@_;
    my $startoutrid=$rid;

    my $next='';

              my $mincond=1;
              my $posnext='';
              if ($direction eq 'forward') {
# --------------------------------------------------------------------- Forward
                  while ($hash{'type_'.$rid} eq 'finish') {
	             $rid=$hash{'ids_'.$hash{'map_id_'.(split(/\./,$rid))[0]}};
                  }
                  map {
                      my $thiscond=
      &Apache::lonnet::directcondval($hash{'condid_'.$hash{'undercond_'.$_}});
                      if ($thiscond>=$mincond) {
		          if ($posnext) {
		             $posnext.=','.$_.':'.$thiscond;
                          } else {
                             $posnext=$_.':'.$thiscond;
		          }
                          if ($thiscond>$mincond) { $mincond=$thiscond; }
	              }
                  } split(/\,/,$hash{'to_'.$rid});
                  map {
                      my ($linkid,$condval)=split(/\:/,$_);
                      if ($condval>=$mincond) {
		          $next=&addrid($next,$hash{'goesto_'.$linkid},
                                $hash{'condid_'.$hash{'undercond_'.$linkid}});
                      }
                  } split(/\,/,$posnext);
                  if ($hash{'is_map_'.$next}) {
# This jumps to the beginning of a new map (going down level)
                      if (
      $hash{'map_type_'.$hash{'map_pc_'.$hash{'src_'.$next}}} eq 'sequence') {
			  $mapurl=$hash{'src_'.$next};
			  $next=$hash{'map_start_'.$hash{'src_'.$next}};
                     }
                  } elsif 
                    ((split(/\./,$startoutrid))[0]!=(split(/\./,$next))[0]) {
# This comes up from a map (coming up one level);
		      $mapurl=$hash{'map_id_'.(split(/\./,$next))[0]};
                  }
              } elsif ($direction eq 'back') {
# ------------------------------------------------------------------- Backwards
                 while ($hash{'type_'.$rid} eq 'start') {
	             $rid=$hash{'ids_'.$hash{'map_id_'.(split(/\./,$rid))[0]}};
		 }
                  map {
                      my $thiscond=
      &Apache::lonnet::directcondval($hash{'condid_'.$hash{'undercond_'.$_}});
                      if ($thiscond>=$mincond) {
		          if ($posnext) {
		             $posnext.=','.$_.':'.$thiscond;
                          } else {
                             $posnext=$_.':'.$thiscond;
		          }
                          if ($thiscond>$mincond) { $mincond=$thiscond; }
	              }
                  } split(/\,/,$hash{'from_'.$rid});
                  map {
                      my ($linkid,$condval)=split(/\:/,$_);
                      if ($condval>=$mincond) {
		          $next=&addrid($next,$hash{'comesfrom_'.$linkid},
                                $hash{'condid_'.$hash{'undercond_'.$linkid}});
                      }
                  } split(/\,/,$posnext);
                  if ($hash{'is_map_'.$next}) {
# This jumps to the end of a new map (going down one level)
                      if (
      $hash{'map_type_'.$hash{'map_pc_'.$hash{'src_'.$next}}} eq 'sequence') {
			  $mapurl=$hash{'src_'.$next};
			  $next=$hash{'map_finish_'.$hash{'src_'.$next}};
                     }
                  } elsif 
                    ((split(/\./,$startoutrid))[0]!=(split(/\./,$next))[0]) {
# This comes back up from a map (going up one level);
		      $mapurl=$hash{'map_id_'.(split(/\./,$next))[0]};
                  }
	      }
              return ($next,$mapurl);
}

# ================================================================ Main Handler

sub handler {
   my $r=shift;

# ------------------------------------------- Set document type for header only

  if ($r->header_only) {
     $r->content_type('text/html');
     $r->send_http_header;
     return OK;
  }

  my %cachehash=(); 
  my $multichoice=0;
  my %multichoicehash=();
  my $redirecturl='';
  my $next='';
  my @possibilities=();
  if (($ENV{'form.postdata'})&&($ENV{'request.course.fn'})) {
      $ENV{'form.postdata'}=~/(\w+)\:(.*)/;
      my $direction=$1;
      my $currenturl=$2;
      if ($direction eq 'return') {
# -------------------------------------------------------- Return to last known
         my $last;
         if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.'_symb.db',
                    &GDBM_READER(),0640)) {
	     $last=$hash{'last_known'};
             untie(%hash);
         }
         my $newloc;
         if ($last) {
            $newloc=&Apache::lonnet::clutter((split(/\_\_\_/,$last))[1]);
         } else {
	    $newloc='/adm/noidea.html';
         }  
	 $r->content_type('text/html');
         $r->header_out(Location => 
			'http://'.$ENV{'HTTP_HOST'}.$newloc);
                               
         return REDIRECT;
      }
      $currenturl=~s/^http\:\/\///;
      $currenturl=~s/^[^\/]+//;
      unless ($currenturl=~/^\/(res|adm\/wrapper|public|adm\/coursedocs)\//) {
	 my $last;
         if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.'_symb.db',
                    &GDBM_READER(),0640)) {
	     $last=$hash{'last_known'};
             untie(%hash);
         }
         if ($last) {
	     $currenturl=&Apache::lonnet::clutter((split(/\_\_\_/,$last))[1]);
	 } else {
	     $r->content_type('text/html');
             $r->header_out(Location => 
                               'http://'.$ENV{'HTTP_HOST'}.'/adm/noidea.html');
             return REDIRECT;
         }
      }
# ------------------------------------------- Do we have any idea where we are?
      my $position;
      if ($position=Apache::lonnet::symbread($currenturl)) {
# ------------------------------------------------------------------------- Yes
	  my ($startoutmap,$mapnum,$thisurl)=split(/\_\_\_/,$position);
          $cachehash{$startoutmap}{$thisurl}=$mapnum;
          $cachehash{$startoutmap}{'last_known'}=
                                       &Apache::lonnet::declutter($currenturl);

# ============================================================ Tie the big hash
          if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.'.db',
                        &GDBM_READER(),0640)) {
              my $rid=$hash{'map_pc_'.&Apache::lonnet::clutter($startoutmap)}.
                      '.'.$mapnum;

# ------------------------------------------------- Move forward, backward, etc
              my $endupmap;
              ($next,$endupmap)=&move($rid,$startoutmap,$direction);
# -------------------------------------- Do we have one and only one empty URL?
              my $safecount=0;
              while (($next) && ($next!~/\,/) && 
                     ((!$hash{'src_'.$next}) || ($hash{'randomout_'.$next}))
                     && ($safecount<10000)) {
                  ($next,$endupmap)=&move($next,$endupmap,$direction);
                  $safecount++;
              }
# We are now at at least one non-empty URL
# ----------------------------------------------------- Check out possibilities
              if ($next) {
                  @possibilities=split(/\,/,$next);
                  if ($#possibilities==0) {
# ---------------------------------------------- Only one possibility, redirect
	              $redirecturl=$hash{'src_'.$next};
                      $cachehash{$endupmap}
                                {&Apache::lonnet::declutter($redirecturl)}
		                                 =(split(/\./,$next))[1];
                  } else {
# ------------------------ There are multiple possibilities for a next resource
                      $multichoice=1;
                      map {
			  $multichoicehash{'src_'.$_}=$hash{'src_'.$_};
                          $multichoicehash{'title_'.$_}=$hash{'title_'.$_};
                          $multichoicehash{'type_'.$_}=$hash{'type_'.$_};
                          (my $first, my $second) = $_ =~ /(\d+).(\d+)/;
                          my $symbSrc = Apache::lonnet::declutter($hash{'src_'.$_});
                          $multichoicehash{'symb_'.$_} = 
                              Apache::lonnet::declutter($hash{'map_id_'.$first}.'___'.
                                                        $second.'___'.$symbSrc);
                                                         
                          my ($choicemap,$choiceres)=split(/\./,$_);
                          $cachehash
			 {&Apache::lonnet::declutter($hash{'src_'.$choicemap})}
                         {&Apache::lonnet::declutter(
						      $multichoicehash
                                                         {'src_'.$_}
                                                       )}
		                                 =$choiceres;
                      } @possibilities;
                  }
	      } else {
# -------------------------------------------------------------- No place to go
                  $multichoice=-1;
              }
# ----------------- The program must come past this point to untie the big hash
	      untie(%hash);
# --------------------------------------------------------- Store position info
              $cachehash{$startoutmap}{'last_direction'}=$direction;
              foreach my $thismap (keys %cachehash) {
                 &Apache::lonnet::symblist($thismap,%{$cachehash{$thismap}});
	      }
# ============================================== Do not return before this line
              if ($redirecturl) {
# ----------------------------------------------------- There is a URL to go to
# ------------------------------------------------- Check for critical messages
		  if ((time-$ENV{'user.criticalcheck.time'})>300) {
                     my @what=&Apache::lonnet::dump
                                  ('critical',$ENV{'user.domain'},
                                              $ENV{'user.name'});
                     if ($what[0]) {
	                if (($what[0] ne 'con_lost') && 
                            ($what[0]!~/^error\:/)) {
	                   $redirecturl='/adm/email?critical=display';
                        }
                     }
                     &Apache::lonnet::appenv('user.criticalcheck.time'=>time);
		  }

		  $r->content_type('text/html');
                  $r->header_out(Location => 
                                'http://'.$ENV{'HTTP_HOST'}.$redirecturl);
                  return REDIRECT;
	      } else {
# --------------------------------------------------------- There was a problem
                  $r->content_type('text/html');
                  $r->send_http_header;
                  if ($#possibilities>0) {
                     $r->print(<<ENDSTART);
<head><title>Choose Next Location</title></head>
<body bgcolor="#FFFFFF">
<h1>LON-CAPA</h1>
There are several possibilities of where to go next.
<p>
Please click on the the resource you intend to access:
<p>
<table border=2>
<tr><th>Title</th><th>Type</th></tr>
ENDSTART
                     foreach (@possibilities) {
                        $r->print(
                              '<tr><td><a href="'.
                              $multichoicehash{'src_'.$_}.'?symb=' .
                                  Apache::lonnet::escape($multichoicehash{'symb_'.$_}).'">'.
                              $multichoicehash{'title_'.$_}.
                              '</a></td><td>'.$multichoicehash{'type_'.$_}.
			      '</td></tr>');
                     }
                     $r->print('</table></body></html>');
		     return OK;
                  } else {
                     $r->print(<<ENDNONE);
<head><title>Choose Next Location</title></head>
<body bgcolor="#FFFFFF">
<img src="/adm/lonIcons/lonlogos.gif" align=right>
<h1>Sorry!</h1>
<h2>Next resource could not be identified.</h2>
<h3>You probably are at the beginning or the end of the course.</h3>
</body>
</html>
ENDNONE
                     return OK;
	         }
	     }
	  } else {
# ------------------------------------------------- Problem, could not tie hash
              $ENV{'user.error.msg'}="/adm/flip:bre:0:1:Course Data Missing";
              return HTTP_NOT_ACCEPTABLE; 
          }
      } else {
# ---------------------------------------- No, could not determine where we are
         $r->internal_redirect('/adm/ambiguous');
      }
  } else {
# -------------------------- Class was not initialized or page fliped strangely
      $ENV{'user.error.msg'}="/adm/flip:bre:0:0:Choose Course";
      return HTTP_NOT_ACCEPTABLE; 
  } 
}

1;
__END__







